# CONQUEST: Large-scale DFT calculations

CONQUEST is a DFT code designed for large-scale calculations, with
excellent parallelisation.  It gives a consistent, exact
diagonalisation approach for systems from 1 to 10,000+ atoms, and
brings the possibility of linear scaling calculations on over
1,000,000 atoms.  The code has been demonstrated on scaling to nearly
200,000 cores and 2,000,000 atoms.

## Capabilities

CONQUEST can perform static electronic structure calculations
(including DOS and band structure calculations), structural
relaxation, molecular dynamics (with NVE, NVT and NPT ensembles all
implemented) and simulation cell optimisation.  It can output energies, forces and stresses as well as
density of states, charge density, orbital density and Tersoff-Hamann
STM images.  The facility to perform delta-SCF and cDFT calculations
is available.  The code can use LibXC for access to a wide variety of
exchange-correlation functionals.

The basis sets used are primarily pseudo-atomic orbitals (PAOs), generated by
a utility provided in the package.  The PAOs can be used as primitive
basis sets, and as basis functions for multi-site support functions (MSSF)
to allow large-scale diagonalisation calculations (up to several
thousand atoms).  Linear scaling calculations with on-site support
functions or small PAO basis sets are also possible.

CONQUEST reads pseudopotentials produced by Don Hamann's
[ONCVPSP](http://www.mat-simresearch.com) code, and is fully
compatible with the the [PseudoDojo](http://www.pseudo-dojo.org) 
and [SG15](http://www.quantum-simulation.org/potentials/sg15_oncv/)
databases. It can also read the .ion
files produced by Siesta, using both the pseudopotentials and
pseudo-atomic orbitals (PAOs) in those files.  CONQUEST integrates with
[ASE](https://wiki.fysik.dtu.dk/ase/index.html), the atomic simulation environment.

The manual is provided with the release (see the ``docs`` directory)
but is also available on
[ReadTheDocs](https://conquest.readthedocs.io/en/latest/). 

## DOI

The DOI for *all* versions of CONQUEST is here; you can find a DOI for each version 
under this umbrella DOI (which will resolve to the latest version).  Please use the 
DOI for the specific version you have used in any given study.

[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.3943720.svg)](https://doi.org/10.5281/zenodo.3943720)
